import { Box, Button, Typography } from '@mui/material'
import axios from 'axios'
import saveAs from 'file-saver'
import JSZip from 'jszip'
import _ from 'lodash'
import { MuiFileInput } from 'mui-file-input'
import { Event, verifyEvent } from 'nostr-tools'
import { useCallback, useEffect, useState } from 'react'
import { useAppSelector } from '../../hooks/store'
import { useLocation, useNavigate, useParams } from 'react-router-dom'
import { toast } from 'react-toastify'
import { LoadingSpinner } from '../../components/LoadingSpinner'
import { NostrController } from '../../controllers'
import { appPublicRoutes } from '../../routes'
import { CreateSignatureEventContent, Meta, SignedEvent } from '../../types'
import {
  decryptArrayBuffer,
  encryptArrayBuffer,
  extractMarksFromSignedMeta,
  extractZipUrlAndEncryptionKey,
  generateEncryptionKey,
  generateKeysFile,
  getCurrentUserFiles,
  getHash,
  hexToNpub,
  isOnline,
  loadZip,
  unixNow,
  npubToHex,
  parseJson,
  readContentOfZipEntry,
  sendNotification,
  signEventForMetaFile,
  updateUsersAppData,
  findOtherUserMarks,
  timeout
} from '../../utils'
import { Container } from '../../components/Container'
import { DisplayMeta } from './internal/displayMeta'
import styles from './style.module.scss'
import { CurrentUserMark, Mark } from '../../types/mark.ts'
import { getLastSignersSig, isFullySigned } from '../../utils/sign.ts'
import {
  filterMarksByPubkey,
  getCurrentUserMarks,
  isCurrentUserMarksComplete,
  updateMarks
} from '../../utils'
import PdfMarking from '../../components/PDFView/PdfMarking.tsx'
import {
  convertToSigitFile,
  getZipWithFiles,
  SigitFile
} from '../../utils/file.ts'
import { ARRAY_BUFFER, DEFLATE } from '../../utils/const.ts'

enum SignedStatus {
  Fully_Signed,
  User_Is_Next_Signer,
  User_Is_Not_Next_Signer
}

export const SignPage = () => {
  const navigate = useNavigate()
  const location = useLocation()
  const params = useParams()

  const usersAppData = useAppSelector((state) => state.userAppData)

  /**
   * Received from `location.state`
   *
   * uploadedZip will be received from home page when a user uploads a sigit zip wrapper that contains keys.json
   * arrayBuffer (decryptedArrayBuffer) will be received in navigation from create page in offline mode
   * meta (metaInNavState) will be received in navigation from create & home page in online mode
   */
  let metaInNavState = location?.state?.meta || undefined
  const { arrayBuffer: decryptedArrayBuffer, uploadedZip } = location.state || {
    decryptedArrayBuffer: undefined,
    uploadedZip: undefined
  }

  /**
   * If userAppData (redux) is available, and we have the route param (sigit id)
   * which is actually a `createEventId`, we will fetch a `sigit`
   * based on the provided route ID and set fetched `sigit` to the `metaInNavState`
   */
  if (usersAppData) {
    const sigitCreateId = params.id

    if (sigitCreateId) {
      const sigit = usersAppData.sigits[sigitCreateId]

      if (sigit) {
        metaInNavState = sigit
      }
    }
  }

  const [displayInput, setDisplayInput] = useState(false)

  const [selectedFile, setSelectedFile] = useState<File | null>(null)

  const [files, setFiles] = useState<{ [filename: string]: SigitFile }>({})

  const [isLoading, setIsLoading] = useState(true)
  const [loadingSpinnerDesc, setLoadingSpinnerDesc] = useState('')

  const [meta, setMeta] = useState<Meta | null>(null)
  const [signedStatus, setSignedStatus] = useState<SignedStatus>()

  const [submittedBy, setSubmittedBy] = useState<string>()

  const [signers, setSigners] = useState<`npub1${string}`[]>([])
  const [viewers, setViewers] = useState<`npub1${string}`[]>([])
  const [marks, setMarks] = useState<Mark[]>([])
  const [creatorFileHashes, setCreatorFileHashes] = useState<{
    [key: string]: string
  }>({})
  const [currentFileHashes, setCurrentFileHashes] = useState<{
    [key: string]: string | null
  }>({})

  const [signedBy, setSignedBy] = useState<`npub1${string}`[]>([])

  const [nextSinger, setNextSinger] = useState<string>()

  // This state variable indicates whether the logged-in user is a signer, a creator, or neither.
  const [isSignerOrCreator, setIsSignerOrCreator] = useState(false)

  const usersPubkey = useAppSelector((state) => state.auth.usersPubkey)

  const nostrController = NostrController.getInstance()
  const [currentUserMarks, setCurrentUserMarks] = useState<CurrentUserMark[]>(
    []
  )
  const [isMarksCompleted, setIsMarksCompleted] = useState(false)
  const [otherUserMarks, setOtherUserMarks] = useState<Mark[]>([])

  useEffect(() => {
    if (signers.length > 0) {
      // check if all signers have signed then its fully signed
      if (isFullySigned(signers, signedBy)) {
        setSignedStatus(SignedStatus.Fully_Signed)
      } else {
        for (const signer of signers) {
          if (!signedBy.includes(signer)) {
            // signers in meta.json are in npub1 format
            // so, convert it to hex before setting to nextSigner
            setNextSinger(npubToHex(signer)!)

            const usersNpub = hexToNpub(usersPubkey!)

            if (signer === usersNpub) {
              // logged in user is the next signer
              setSignedStatus(SignedStatus.User_Is_Next_Signer)
            } else {
              setSignedStatus(SignedStatus.User_Is_Not_Next_Signer)
            }

            break
          }
        }
      }
    } else {
      // there's no signer just viewers. So its fully signed
      setSignedStatus(SignedStatus.Fully_Signed)
    }

    // Determine and set the status of the user
    if (submittedBy && usersPubkey && submittedBy === usersPubkey) {
      // If the submission was made by the user, set the status to true
      setIsSignerOrCreator(true)
    } else if (usersPubkey) {
      // Convert the user's public key from hex to npub format
      const usersNpub = hexToNpub(usersPubkey)
      if (signers.includes(usersNpub)) {
        // If the user's npub is in the list of signers, set the status to true
        setIsSignerOrCreator(true)
      }
    }
  }, [signers, signedBy, usersPubkey, submittedBy])

  useEffect(() => {
    const handleUpdatedMeta = async (meta: Meta) => {
      const createSignatureEvent = await parseJson<Event>(
        meta.createSignature
      ).catch((err) => {
        console.log('err in parsing the createSignature event:>> ', err)
        toast.error(
          err.message || 'error occurred in parsing the create signature event'
        )
        setIsLoading(false)
        return null
      })

      if (!createSignatureEvent) return

      const isValidCreateSignature = verifyEvent(createSignatureEvent)

      if (!isValidCreateSignature) {
        toast.error('Create signature is invalid')
        setIsLoading(false)
        return
      }

      const createSignatureContent =
        await parseJson<CreateSignatureEventContent>(
          createSignatureEvent.content
        ).catch((err) => {
          console.log(
            `err in parsing the createSignature event's content :>> `,
            err
          )
          toast.error(
            err.message ||
              `error occurred in parsing the create signature event's content`
          )
          setIsLoading(false)
          return null
        })

      if (!createSignatureContent) return

      setSigners(createSignatureContent.signers)
      setViewers(createSignatureContent.viewers)
      setCreatorFileHashes(createSignatureContent.fileHashes)
      setSubmittedBy(createSignatureEvent.pubkey)
      setMarks(createSignatureContent.markConfig)

      if (usersPubkey) {
        const metaMarks = filterMarksByPubkey(
          createSignatureContent.markConfig,
          usersPubkey!
        )
        const signedMarks = extractMarksFromSignedMeta(meta)
        const currentUserMarks = getCurrentUserMarks(metaMarks, signedMarks)
        const otherUserMarks = findOtherUserMarks(signedMarks, usersPubkey!)
        setOtherUserMarks(otherUserMarks)
        setCurrentUserMarks(currentUserMarks)
        setIsMarksCompleted(isCurrentUserMarksComplete(currentUserMarks))
      }

      setSignedBy(Object.keys(meta.docSignatures) as `npub1${string}`[])
    }

    if (meta) {
      handleUpdatedMeta(meta)
    }
  }, [meta, usersPubkey])

  const handleDownload = async () => {
    if (Object.entries(files).length === 0 || !meta || !usersPubkey) return
    setLoadingSpinnerDesc('Generating file')
    try {
      const zip = await getZipWithFiles(meta, files)
      const arrayBuffer = await zip.generateAsync({
        type: ARRAY_BUFFER,
        compression: DEFLATE,
        compressionOptions: {
          level: 6
        }
      })
      if (!arrayBuffer) return
      const blob = new Blob([arrayBuffer])
      saveAs(blob, `exported-${unixNow()}.sigit.zip`)
    } catch (error) {
      console.log('error in zip:>> ', error)
      if (error instanceof Error) {
        toast.error(error.message || 'Error occurred in generating zip file')
      }
    }
  }

  const decrypt = useCallback(
    async (file: File) => {
      setLoadingSpinnerDesc('Decrypting file')

      const zip = await loadZip(file)
      if (!zip) return

      const parsedKeysJson = await parseKeysJson(zip)
      if (!parsedKeysJson) return

      const encryptedArrayBuffer = await readContentOfZipEntry(
        zip,
        'compressed.sigit',
        'arraybuffer'
      )

      if (!encryptedArrayBuffer) return

      const { keys, sender } = parsedKeysJson

      for (const key of keys) {
        // decrypt the encryptionKey, with timeout (duration = 60 seconds)
        const encryptionKey = await Promise.race([
          nostrController.nip04Decrypt(sender, key),
          timeout(60000)
        ])
          .then((res) => {
            return res
          })
          .catch((err) => {
            console.log('err :>> ', err)
            return null
          })

        // Return if encryption failed
        if (!encryptionKey) continue

        const arrayBuffer = await decryptArrayBuffer(
          encryptedArrayBuffer,
          encryptionKey
        )
          .catch((err) => {
            console.log('err in decryption:>> ', err)
            return null
          })
          .finally(() => {
            setIsLoading(false)
          })

        if (arrayBuffer) return arrayBuffer
      }

      return null
    },
    [nostrController]
  )

  useEffect(() => {
    // online mode - from create and home page views
    if (metaInNavState) {
      const processSigit = async () => {
        setIsLoading(true)
        setLoadingSpinnerDesc('Extracting zipUrl and encryption key from meta')

        const res = await extractZipUrlAndEncryptionKey(metaInNavState)
        if (!res) {
          setIsLoading(false)
          return
        }

        const { zipUrl, encryptionKey } = res

        setLoadingSpinnerDesc('Fetching file from file server')
        axios
          .get(zipUrl, {
            responseType: 'arraybuffer'
          })
          .then((res) => {
            handleArrayBufferFromBlossom(res.data, encryptionKey)
            setMeta(metaInNavState)
          })
          .catch((err) => {
            console.error(`error occurred in getting file from ${zipUrl}`, err)
            toast.error(
              err.message || `error occurred in getting file from ${zipUrl}`
            )
          })
          .finally(() => {
            setIsLoading(false)
          })
      }

      processSigit()
    } else if (decryptedArrayBuffer) {
      handleDecryptedArrayBuffer(decryptedArrayBuffer).finally(() =>
        setIsLoading(false)
      )
    } else if (uploadedZip) {
      decrypt(uploadedZip)
        .then((arrayBuffer) => {
          if (arrayBuffer) handleDecryptedArrayBuffer(arrayBuffer)
        })
        .catch((err) => {
          console.error(`error occurred in decryption`, err)
          toast.error(err.message || `error occurred in decryption`)
        })
        .finally(() => {
          setIsLoading(false)
        })
    } else {
      setIsLoading(false)
      setDisplayInput(true)
    }
  }, [decryptedArrayBuffer, uploadedZip, metaInNavState, decrypt])

  const handleArrayBufferFromBlossom = async (
    arrayBuffer: ArrayBuffer,
    encryptionKey: string
  ) => {
    // array buffer returned from blossom is encrypted.
    // So, first decrypt it
    const decrypted = await decryptArrayBuffer(
      arrayBuffer,
      encryptionKey
    ).catch((err) => {
      console.log('err in decryption:>> ', err)
      toast.error(err.message || 'An error occurred in decrypting file.')
      setIsLoading(false)
      return null
    })

    if (!decrypted) return

    const zip = await loadZip(decrypted)
    if (!zip) {
      setIsLoading(false)
      return
    }

    const files: { [filename: string]: SigitFile } = {}
    const fileHashes: { [key: string]: string | null } = {}
    const fileNames = Object.values(zip.files).map((entry) => entry.name)

    // generate hashes for all files in zipArchive
    // these hashes can be used to verify the originality of files
    for (const fileName of fileNames) {
      const arrayBuffer = await readContentOfZipEntry(
        zip,
        fileName,
        'arraybuffer'
      )

      if (arrayBuffer) {
        files[fileName] = await convertToSigitFile(arrayBuffer, fileName)
        const hash = await getHash(arrayBuffer)
        if (hash) {
          fileHashes[fileName] = hash
        }
      } else {
        fileHashes[fileName] = null
      }
    }

    setFiles(files)
    setCurrentFileHashes(fileHashes)
  }

  const setUpdatedMarks = (markToUpdate: Mark) => {
    const updatedMarks = updateMarks(marks, markToUpdate)
    setMarks(updatedMarks)
  }

  const parseKeysJson = async (zip: JSZip) => {
    const keysFileContent = await readContentOfZipEntry(
      zip,
      'keys.json',
      'string'
    )

    if (!keysFileContent) return null

    return await parseJson<{ sender: string; keys: string[] }>(
      keysFileContent
    ).catch((err) => {
      console.log(`Error parsing content of keys.json:`, err)
      toast.error(err.message || `Error parsing content of keys.json`)
      return null
    })
  }

  const handleDecryptedArrayBuffer = async (arrayBuffer: ArrayBuffer) => {
    const decryptedZipFile = new File([arrayBuffer], 'decrypted.zip')

    setLoadingSpinnerDesc('Parsing zip file')

    const zip = await loadZip(decryptedZipFile)
    if (!zip) return

    const files: { [filename: string]: SigitFile } = {}
    const fileHashes: { [key: string]: string | null } = {}
    const fileNames = Object.values(zip.files)
      .filter((entry) => entry.name.startsWith('files/') && !entry.dir)
      .map((entry) => entry.name)

    for (const zipFilePath of fileNames) {
      const arrayBuffer = await readContentOfZipEntry(
        zip,
        zipFilePath,
        'arraybuffer'
      )

      const fileName = zipFilePath.replace(/^files\//, '')
      if (arrayBuffer) {
        files[fileName] = await convertToSigitFile(arrayBuffer, fileName)

        // generate hashes for all entries in files folder of zipArchive
        // these hashes can be used to verify the originality of files
        const hash = await getHash(arrayBuffer)
        if (hash) {
          fileHashes[fileName] = hash
        }
      } else {
        fileHashes[fileName] = null
      }
    }

    setFiles(files)
    setCurrentFileHashes(fileHashes)

    setDisplayInput(false)

    setLoadingSpinnerDesc('Parsing meta.json')

    const metaFileContent = await readContentOfZipEntry(
      zip,
      'meta.json',
      'string'
    )

    if (!metaFileContent) {
      setIsLoading(false)
      return
    }

    const parsedMetaJson = await parseJson<Meta>(metaFileContent).catch(
      (err) => {
        console.log('err in parsing the content of meta.json :>> ', err)
        toast.error(
          err.message || 'error occurred in parsing the content of meta.json'
        )
        setIsLoading(false)
        return null
      }
    )

    setMeta(parsedMetaJson)
  }

  const handleDecrypt = async () => {
    if (!selectedFile) return

    setIsLoading(true)
    const arrayBuffer = await decrypt(selectedFile)

    if (!arrayBuffer) return

    handleDecryptedArrayBuffer(arrayBuffer)
  }

  const handleSign = async () => {
    if (Object.entries(files).length === 0 || !meta) return

    setIsLoading(true)

    setLoadingSpinnerDesc('Signing nostr event')

    const prevSig = getPrevSignersSig(hexToNpub(usersPubkey!))
    if (!prevSig) {
      setIsLoading(false)
      toast.error('Previous signature is invalid')
      return
    }

    const marks = getSignerMarksForMeta() || []

    const signedEvent = await signEventForMeta({ prevSig, marks })
    if (!signedEvent) return

    const updatedMeta = updateMetaSignatures(meta, signedEvent)

    if (await isOnline()) {
      await handleOnlineFlow(updatedMeta)
    } else {
      setMeta(updatedMeta)
      setIsLoading(false)
    }
  }

  // Sign the event for the meta file
  const signEventForMeta = async (signerContent: {
    prevSig: string
    marks: Mark[]
  }) => {
    return await signEventForMetaFile(
      JSON.stringify(signerContent),
      nostrController,
      setIsLoading
    )
  }

  const getSignerMarksForMeta = (): Mark[] | undefined => {
    if (currentUserMarks.length === 0) return
    return currentUserMarks.map(({ mark }: CurrentUserMark) => mark)
  }

  // Update the meta signatures
  const updateMetaSignatures = (meta: Meta, signedEvent: SignedEvent): Meta => {
    const metaCopy = _.cloneDeep(meta)
    metaCopy.docSignatures = {
      ...metaCopy.docSignatures,
      [hexToNpub(signedEvent.pubkey)]: JSON.stringify(signedEvent, null, 2)
    }
    metaCopy.modifiedAt = unixNow()
    return metaCopy
  }

  // create final zip file
  const createFinalZipFile = async (
    encryptedArrayBuffer: ArrayBuffer,
    encryptionKey: string
  ): Promise<File | null> => {
    // Get the current timestamp in seconds
    const blob = new Blob([encryptedArrayBuffer])
    // Create a File object with the Blob data
    const file = new File([blob], `compressed.sigit`, {
      type: 'application/sigit'
    })

    const isLastSigner = checkIsLastSigner(signers)

    const userSet = new Set<string>()

    if (isLastSigner) {
      if (submittedBy) {
        userSet.add(submittedBy)
      }

      signers.forEach((signer) => {
        userSet.add(npubToHex(signer)!)
      })

      viewers.forEach((viewer) => {
        userSet.add(npubToHex(viewer)!)
      })
    } else {
      const usersNpub = hexToNpub(usersPubkey!)
      const signerIndex = signers.indexOf(usersNpub)
      const nextSigner = signers[signerIndex + 1]
      userSet.add(npubToHex(nextSigner)!)
    }

    const keysFileContent = await generateKeysFile(
      Array.from(userSet),
      encryptionKey
    )
    if (!keysFileContent) return null

    const zip = new JSZip()
    zip.file(`compressed.sigit`, file)
    zip.file('keys.json', keysFileContent)

    const arraybuffer = await zip
      .generateAsync({
        type: 'arraybuffer',
        compression: 'DEFLATE',
        compressionOptions: { level: 6 }
      })
      .catch(handleZipError)

    if (!arraybuffer) return null

    return new File([new Blob([arraybuffer])], `${unixNow()}.sigit.zip`, {
      type: 'application/zip'
    })
  }

  // Handle errors during zip file generation
  const handleZipError = (err: unknown) => {
    console.log('Error in zip:>> ', err)
    setIsLoading(false)
    if (err instanceof Error) {
      toast.error(err.message || 'Error occurred in generating zip file')
    }
    return null
  }

  // Handle the online flow: update users app data and send notifications
  const handleOnlineFlow = async (meta: Meta) => {
    setLoadingSpinnerDesc('Updating users app data')
    const updatedEvent = await updateUsersAppData(meta)
    if (!updatedEvent) {
      setIsLoading(false)
      return
    }

    const userSet = new Set<`npub1${string}`>()
    if (submittedBy && submittedBy !== usersPubkey) {
      userSet.add(hexToNpub(submittedBy))
    }

    const usersNpub = hexToNpub(usersPubkey!)
    const isLastSigner = checkIsLastSigner(signers)
    if (isLastSigner) {
      signers.forEach((signer) => {
        if (signer !== usersNpub) {
          userSet.add(signer)
        }
      })

      viewers.forEach((viewer) => {
        userSet.add(viewer)
      })
    } else {
      const currentSignerIndex = signers.indexOf(usersNpub)
      const prevSigners = signers.slice(0, currentSignerIndex)

      prevSigners.forEach((signer) => {
        userSet.add(signer)
      })

      const nextSigner = signers[currentSignerIndex + 1]
      userSet.add(nextSigner)
    }

    setLoadingSpinnerDesc('Sending notifications')
    const users = Array.from(userSet)
    const promises = users.map((user) =>
      sendNotification(npubToHex(user)!, meta)
    )
    await Promise.all(promises)
      .then(() => {
        toast.success('Notifications sent successfully')
        setMeta(meta)
      })
      .catch(() => {
        toast.error('Failed to publish notifications')
      })

    setIsLoading(false)
  }

  // Check if the current user is the last signer
  const checkIsLastSigner = (signers: string[]): boolean => {
    const usersNpub = hexToNpub(usersPubkey!)
    const lastSignerIndex = signers.length - 1
    const signerIndex = signers.indexOf(usersNpub)
    return signerIndex === lastSignerIndex
  }

  const handleExport = async () => {
    if (Object.entries(files).length === 0 || !meta || !usersPubkey) return

    const usersNpub = hexToNpub(usersPubkey)
    if (
      !signers.includes(usersNpub) &&
      !viewers.includes(usersNpub) &&
      submittedBy !== usersNpub
    )
      return

    setIsLoading(true)
    setLoadingSpinnerDesc('Signing nostr event')

    if (!meta) return

    const prevSig = getLastSignersSig(meta, signers)
    if (!prevSig) return

    const signedEvent = await signEventForMetaFile(
      JSON.stringify({
        prevSig
      }),
      nostrController,
      setIsLoading
    )

    if (!signedEvent) return

    const exportSignature = JSON.stringify(signedEvent, null, 2)

    const stringifiedMeta = JSON.stringify(
      {
        ...meta,
        exportSignature
      },
      null,
      2
    )

    const zip = new JSZip()

    zip.file('meta.json', stringifiedMeta)

    for (const [fileName, file] of Object.entries(files)) {
      zip.file(`files/${fileName}`, await file.arrayBuffer())
    }

    const arrayBuffer = await zip
      .generateAsync({
        type: 'arraybuffer',
        compression: 'DEFLATE',
        compressionOptions: {
          level: 6
        }
      })
      .catch((err) => {
        console.log('err in zip:>> ', err)
        setIsLoading(false)
        toast.error(err.message || 'Error occurred in generating zip file')
        return null
      })

    if (!arrayBuffer) return

    const blob = new Blob([arrayBuffer])
    saveAs(blob, `exported-${unixNow()}.sigit.zip`)

    setIsLoading(false)

    navigate(appPublicRoutes.verify)
  }

  const handleExportSigit = async () => {
    if (Object.entries(files).length === 0 || !meta) return

    const zip = new JSZip()

    const stringifiedMeta = JSON.stringify(meta, null, 2)

    zip.file('meta.json', stringifiedMeta)

    for (const [fileName, file] of Object.entries(files)) {
      zip.file(`files/${fileName}`, await file.arrayBuffer())
    }

    const arrayBuffer = await zip
      .generateAsync({
        type: 'arraybuffer',
        compression: 'DEFLATE',
        compressionOptions: {
          level: 6
        }
      })
      .catch((err) => {
        console.log('err in zip:>> ', err)
        setIsLoading(false)
        toast.error(err.message || 'Error occurred in generating zip file')
        return null
      })

    if (!arrayBuffer) return

    const key = await generateEncryptionKey()

    setLoadingSpinnerDesc('Encrypting zip file')
    const encryptedArrayBuffer = await encryptArrayBuffer(arrayBuffer, key)

    const finalZipFile = await createFinalZipFile(encryptedArrayBuffer, key)

    if (!finalZipFile) return
    saveAs(finalZipFile, `exported-${unixNow()}.sigit.zip`)
  }

  /**
   * This function accepts an npub of a signer and return the signature of its previous signer.
   * This prevSig will be used in the content of the provided signer's signedEvent
   */
  const getPrevSignersSig = (npub: string) => {
    if (!meta) return null

    // if user is first signer then use creator's signature
    if (signers[0] === npub) {
      try {
        const createSignatureEvent: Event = JSON.parse(meta.createSignature)
        return createSignatureEvent.sig
      } catch (error) {
        return null
      }
    }

    // find the index of signer
    const currentSignerIndex = signers.findIndex((signer) => signer === npub)
    // return null if could not found user in signer's list
    if (currentSignerIndex === -1) return null
    // find prev signer
    const prevSigner = signers[currentSignerIndex - 1]

    // get the signature of prev signer
    try {
      const prevSignersEvent: Event = JSON.parse(meta.docSignatures[prevSigner])
      return prevSignersEvent.sig
    } catch (error) {
      return null
    }
  }

  if (isLoading) {
    return <LoadingSpinner desc={loadingSpinnerDesc} />
  }

  if (!isMarksCompleted && signedStatus === SignedStatus.User_Is_Next_Signer) {
    return (
      <PdfMarking
        files={getCurrentUserFiles(files, currentFileHashes, creatorFileHashes)}
        currentUserMarks={currentUserMarks}
        setIsMarksCompleted={setIsMarksCompleted}
        setCurrentUserMarks={setCurrentUserMarks}
        setUpdatedMarks={setUpdatedMarks}
        handleDownload={handleDownload}
        otherUserMarks={otherUserMarks}
        meta={meta}
      />
    )
  }

  return (
    <>
      <Container className={styles.container}>
        {displayInput && (
          <>
            <Typography component="label" variant="h6">
              Select sigit file
            </Typography>

            <Box className={styles.inputBlock}>
              <MuiFileInput
                placeholder="Select file"
                inputProps={{ accept: '.sigit.zip' }}
                value={selectedFile}
                onChange={(value) => setSelectedFile(value)}
              />
            </Box>

            {selectedFile && (
              <Box sx={{ mt: 2, display: 'flex', justifyContent: 'center' }}>
                <Button onClick={handleDecrypt} variant="contained">
                  Decrypt
                </Button>
              </Box>
            )}
          </>
        )}

        {submittedBy && Object.entries(files).length > 0 && meta && (
          <>
            <DisplayMeta
              meta={meta}
              files={files}
              submittedBy={submittedBy}
              signers={signers}
              viewers={viewers}
              creatorFileHashes={creatorFileHashes}
              currentFileHashes={currentFileHashes}
              signedBy={signedBy}
              nextSigner={nextSinger}
              getPrevSignersSig={getPrevSignersSig}
            />

            {signedStatus === SignedStatus.Fully_Signed && (
              <Box sx={{ mt: 1, display: 'flex', justifyContent: 'center' }}>
                <Button onClick={handleExport} variant="contained">
                  Export
                </Button>
              </Box>
            )}

            {signedStatus === SignedStatus.User_Is_Next_Signer && (
              <Box sx={{ mt: 1, display: 'flex', justifyContent: 'center' }}>
                <Button onClick={handleSign} variant="contained">
                  Sign
                </Button>
              </Box>
            )}

            {isSignerOrCreator && (
              <Box sx={{ mt: 1, display: 'flex', justifyContent: 'center' }}>
                <Button onClick={handleExportSigit} variant="contained">
                  Export Sigit
                </Button>
              </Box>
            )}
          </>
        )}
      </Container>
    </>
  )
}
